@@ -492,7 +492,7 @@ sub parse_module {
         return $maybe if IS_MODOBJ->( module => $maybe );
     }
 
-    ### Special case arbitary file paths such as '.' etc.
+    ### Special case arbitrary file paths such as '.' etc.
     if ( $mod and -d File::Spec->rel2abs($mod) ) {
         my $dir    = File::Spec->rel2abs($mod);
         my $parent = File::Spec->rel2abs( File::Spec->catdir( $dir, '..' ) );
@@ -729,6 +729,15 @@ sub parse_module {
     ### let's start putting the blame somewhere
     } else {
 
+        # Lets not give up too easily. There is one last chance
+        # http://perlmonks.org/?node_id=805957
+        # This should catch edge-cases where the package name
+        # is unrelated to the modules it contains.
+
+        my ($modobj) = grep { $_->package_name eq $mod } 
+                        $self->search( type => 'package', allow => [ qr/^\Q$mod\E/ ], );
+        return $modobj if IS_MODOBJ->( module => $modobj );
+
         unless( $author ) {
             error( loc( "'%1' does not contain an author part", $mod ) );
         }
@@ -1153,7 +1162,8 @@ sub save_state {
 
 
 ### XXX these wrappers are not individually tested! only the underlying
-### code through source.t and indirectly trought he CustomSource plugin.
+### code through source.t and indirectly through he CustomSource plugin.
+
 =pod
 
 =head1 CUSTOM MODULE SOURCES
