@@ -20,7 +20,7 @@ BEGIN {
   #require AutoLoader;
 
   our @ISA = qw(Exporter);
-  our $VERSION = "1.03";
+  our $VERSION = "1.05";
   XSLoader::load('OS2::Process', $VERSION);
 }
 
@@ -332,7 +332,7 @@ my $swentry_size = swentry_size();
 sub sw_entries () {
   my $s = swentries_list();
   my ($c, $s1) = unpack 'La*', $s;
-  die "Unconsistent size in swentries_list()" unless 4+$c*$swentry_size == length $s;
+  die "Inconsistent size in swentries_list()" unless 4+$c*$swentry_size == length $s;
   my (@l, $e);
   push @l, $e while $e = substr $s1, 0, $swentry_size, '';
   @l;
@@ -1070,44 +1070,44 @@ returns a list of the following data:
 
 =over
 
-=item
+=item *
 
 Title of the process (in the C<Ctrl-Esc> list);
 
-=item
+=item *
 
 window handle of switch entry of the process (in the C<Ctrl-Esc> list);
 
-=item
+=item *
 
 window handle of the icon of the process;
 
-=item
+=item *
 
 process handle of the owner of the entry in C<Ctrl-Esc> list;
 
-=item
+=item *
 
 process id of the owner of the entry in C<Ctrl-Esc> list;
 
-=item
+=item *
 
 session id of the owner of the entry in C<Ctrl-Esc> list;
 
-=item
+=item *
 
 whether visible in C<Ctrl-Esc> list;
 
-=item
+=item *
 
 whether item cannot be switched to (note that it is not actually
 grayed in the C<Ctrl-Esc> list));
 
-=item
+=item *
 
 whether participates in jump sequence;
 
-=item
+=item *
 
 program type.  Possible values are:
 
@@ -1127,7 +1127,7 @@ is a windowed WIN-OS/2 program, it runs in a PROG_WINDOWEDVDM
 session. Likewise, if it's a full-screen WIN-OS/2 program, it runs in
 a PROG_VDM session.
 
-=item
+=item *
 
 switch-entry handle.
 
@@ -1225,7 +1225,7 @@ Some of these API's require sending a message to the specified window.
 In such a case the process needs to be a PM process, or to be morphed
 to a PM process via OS2::MorphPM().
 
-For a temporary morphing to PM use L<OS2::localMorphPM class>.
+For a temporary morphing to PM use the L<OS2::localMorphPM> class.
 
 Keep in mind that PM windows are engaged in 2 "orthogonal" window
 trees, as well as in the z-order list.
@@ -1234,7 +1234,7 @@ One tree is given by the I<parent/child> relationship.  This
 relationship affects drawing (child is drawn relative to its parent
 (lower-left corner), and the drawing is clipped by the parent's
 boundary; parent may request that I<it's> drawing is clipped to be
-confined to the outsize of the childs and/or siblings' windows);
+confined to the outsize of the child's and/or siblings' windows);
 hiding; minimizing/restoring; and destroying windows.
 
 Another tree (not necessarily connected?) is given by I<ownership>
@@ -1294,7 +1294,7 @@ is governed by $flags.
 
 Same as C<WindowPos_set>, but takes the position from keys C<fl width height
 x y behind hwnd> of the hash referenced by $hash.  If $hwnd is explicitly
-specified, it overrides C<$hash->{hwnd}>.  If $hash->{flags} is not specified,
+specified, it overrides C<< $hash->{hwnd} >>.  If $hash->{flags} is not specified,
 it is calculated basing on the existing keys of $hash.  Requires (morphing to) PM.
 
 Example:
@@ -1388,7 +1388,7 @@ state" do not change the appearance of the window.  Default: $update is TRUE.
 Set the window enabled state.  Default: $enable is TRUE.
 
 Results in C<WM_ENABLED> message sent to the window.  Typically, this
-would change the appearence of the window.  If at the moment of disabling
+would change the appearance of the window.  If at the moment of disabling
 focus is in the window (or a descendant), focus is lost (no focus anywhere).
 If focus is needed, it can be reassigned explicitly later.
 
@@ -1737,19 +1737,19 @@ $addr should be a memory address (encoded as integer).  This call finds
 the largest continuous region of memory belonging to the same memory object
 as $addr, and having the same memory flags as $addr. $flags is the value of
 the memory flag of $addr (see docs of DosQueryMem(3) for details).  If
-optional argumetn $size_lim is given, the search is restricted to the region
+optional argument $size_lim is given, the search is restricted to the region
 this many bytes long (after $addr).
 
 ($addr and $size are rounded so that all the memory pages containing
 the region are inspected.)  Optional argument $interrupt (defaults to 1)
-specifies whether region scan should be interruptable by signals.
+specifies whether region scan should be interruptible by signals.
 
 =back
 
 Use class C<OS2::localClipbrd> to ensure that clipboard is closed even if
 the code in the block made a non-local exit.
 
-See L<"OS2::localMorphPM and OS2::localClipbrd classes">.
+See the L<OS2::localMorphPM> and L<OS2::localClipbrd> classes.
 
 =head2 Control of the PM atom tables
 
@@ -1909,7 +1909,7 @@ With C<MB_APPLMODAL> the owner of the dialogue is disabled; therefore, do not
 specify the owner as the parent if this option is used.
 
 Additionally, the following flag is possible, but probably not very useful:
-  
+
  Help button 
      MB_HELP             a HELP button appears, which sends a WM_HELP
 				 message is sent to the window procedure of the
@@ -2083,7 +2083,7 @@ For direct access, see also the L<"EXPORTS"> section; the latter way
 may also provide some performance advantages, since the value of the
 constant is cached.
 
-=head1 OS2::localMorphPM, OS2::localFlashWindow, and OS2::localClipbrd classes
+=head1 L<OS2::localMorphPM>, OS2::localFlashWindow, and OS2::localClipbrd classes
 
 The class C<OS2::localMorphPM> morphs the process to PM for the duration of
 the given scope.
@@ -2106,7 +2106,7 @@ morph the application into PM:
   }
 
 C<OS2::localFlashWindow> behaves similarly; see
-L<"FlashWindow($hwnd,$doFlash)">.
+L<FlashWindow($hwnd, $doFlash)>.
 
 =head1 EXAMPLES
 
